@@ -1,8 +1,15 @@
-# -*- ruby -*-
+# -*- mode: ruby; coding: us-ascii; -*-
 
 require 'rubygems'
 require 'rake'
 require 'rake/testtask'
+require 'rbconfig'
+
+begin
+  require 'psych'
+rescue LoadError
+  # do nothing
+end
 require 'yaml'
 
 require 'hoe/rake'
@@ -57,8 +64,11 @@ require 'hoe/rake'
 #   end
 
 class Hoe
+
+  include Rake::DSL if defined?(Rake::DSL)
+
   # duh
-  VERSION = '2.8.0'
+  VERSION = '2.12.3'
 
   @@plugins = [:clean, :debug, :deps, :flay, :flog, :newb, :package,
                :publish, :rcov, :gemcutter, :signing, :test]
@@ -87,7 +97,7 @@ class Hoe
   ##
   # True if you're a masochistic developer. Used for building commands.
 
-  WINDOZE = /mswin|mingw/ =~ RUBY_PLATFORM unless defined? WINDOZE
+  WINDOZE = RbConfig::CONFIG['host_os'] =~ /mswin|mingw/ unless defined? WINDOZE
 
   ##
   # *MANDATORY*: The author(s) of the package. (can be array)
@@ -202,12 +212,22 @@ class Hoe
   attr_accessor :test_globs
 
   ##
-  # Optional: The url(s) of the project. (can be array).
+  # Deprecated: Optional: The url(s) of the project. (can be array).
   # Auto-populates to a list of urls read from the beginning of
   # README.txt.
+  #
 
   attr_accessor :url
 
+  ##
+  # Optional: The urls of the project. This can be an array or
+  # (preferably) a hash. Auto-populates to the urls read from the
+  # beginning of README.txt.
+  #
+  # See parse_urls for more details
+
+  attr_accessor :urls
+
   ##
   # *MANDATORY*: The version. Don't hardcode! use a constant in the project.
 
@@ -228,48 +248,70 @@ class Hoe
   #
   # It is called at the end of hoe.rb
 
-  def self.load_plugins
-    loaded, found = {}, {}
+  def self.load_plugins plugins = Hoe.plugins
+    @found  ||= {}
+    @loaded ||= {}
+    @files  ||= Gem.find_files "hoe/*.rb"
 
-    Gem.find_files("hoe/*.rb").reverse.each do |path|
-      found[File.basename(path, ".rb").intern] = path
+    @files.reverse.each do |path|
+      @found[File.basename(path, ".rb").intern] = path
     end
 
-    :keep_doing_this while found.map { |name, plugin|
-      next unless Hoe.plugins.include? name
-      next if loaded[name]
+    :keep_doing_this while @found.map { |name, plugin|
+      next unless plugins.include? name
+      next if @loaded[name]
       begin
         warn "loading #{plugin}" if $DEBUG
-        loaded[name] = load plugin
+        @loaded[name] = require plugin
       rescue LoadError => e
         warn "error loading #{plugin.inspect}: #{e.message}. skipping..."
       end
     }.any?
+
+    bad_plugins = plugins - @loaded.keys
+    bad_plugins.each do |bad_plugin|
+      plugins.delete bad_plugin
+    end
+
+    return @loaded, @found
   end
 
   ##
-  # Normalize a project name into the project, file, and klass cases (?!?).
+  # Normalize a project name into the project, file, and klass names that
+  # follow Ruby package naming guidelines.
   #
-  # no, I have no idea how to describe this. Does the thing with the stuff.
+  # Project names are lowercase with _ separating package parts and -
+  # separating extension parts.
+  #
+  # File names are lowercase with _ separating pacagke parts and / separating
+  # extension parts.  net-http-persistent becomes net/http/persistent.
+  #
+  # Klass names are CamelCase with :: separating extension parts.
 
   def self.normalize_names project # :nodoc:
-    project   = project.tr('-', '_').gsub(/([A-Z])/, '_\1').downcase.sub(/^_/, '')
+    project   = project.gsub(/([A-Z])/, '_\1').downcase.sub(/^_/, '')
     klass     = project.gsub(/(?:^|_)([a-z])/) { $1.upcase }
-    file_name = project
+    klass     = klass.  gsub(/(?:^|-)([a-z])/) { "::#{$1.upcase}" }
+    file_name = project.gsub(/-/, '/')
 
     return project, file_name, klass
   end
 
   ##
-  # Activate plugins.
+  # Activates +plugins+.  If a plugin cannot be loaded it will be ignored.
+  #
+  # Plugins may also be activated through a +plugins+ array in
+  # <tt>~/.hoerc</tt>.  This should only be used for plugins that aren't
+  # critical to your project and plugins that you want to use on other
+  # projects.
 
-  def self.plugin *names
-    self.plugins.concat names
+  def self.plugin *plugins
+    self.plugins.concat plugins
     self.plugins.uniq!
   end
 
   ##
-  # Return the list of activated plugins.
+  # The list of active plugins.
 
   def self.plugins
     @@plugins
@@ -298,7 +340,15 @@ class Hoe
   # Activate plugin modules and add them to the current instance.
 
   def activate_plugins
-    names = self.class.constants.map { |s| s.to_s }
+    with_config do |config, _|
+      config_plugins = config['plugins']
+      break unless config_plugins
+      Hoe.plugins.concat config_plugins.map { |plugin| plugin.intern }
+    end
+
+    Hoe.load_plugins Hoe.plugins
+
+    names = Hoe.constants.map { |s| s.to_s }
     names.reject! { |n| n =~ /^[A-Z_]+$/ }
 
     names.each do |name|
@@ -314,6 +364,34 @@ class Hoe
     end
   end
 
+  ##
+  # Add a dependency declaration to your spec. Pass :dev to
+  # +type+ for developer dependencies.
+
+  def dependency name, version, type = :runtime
+    raise "Unknown dependency type: #{type}" unless
+      [:runtime, :dev, :development, :developer].include? type
+
+    # spec has already been defined. A task wants to add a dependency after.
+    if spec then
+      msg = if type == :runtime then
+              :add_dependency
+            else
+              :add_development_dependency
+            end
+
+      spec.send msg, name, version
+    else
+      ary = if type == :runtime then
+              extra_deps
+            else
+              extra_dev_deps
+            end
+
+      ary << [name, version]
+    end
+  end
+
   ##
   # Add standard and user defined dependencies to the spec.
 
@@ -323,11 +401,10 @@ class Hoe
 
     case name
     when 'hoe' then
-      extra_deps << ['rake', ">= #{RAKEVERSION}"]
-    when 'rubyforge', 'rake', 'gemcutter' then
-      # avoid circular dependencies for hoe's (potentially) hoe'd dependencies
+      dependency "rake", "~> 0.8"
     else
-      extra_dev_deps << ['hoe', ">= #{VERSION}"]
+      version = VERSION.split(/\./).first(2).join(".")
+      dependency "hoe", "~> #{version}", :development
     end
   end
 
@@ -345,6 +422,15 @@ class Hoe
     self.spec = Gem::Specification.new do |s|
       dirs = Dir['lib']
 
+      manifest = File.read_utf("Manifest.txt").split(/\r?\n\r?/) rescue nil
+
+      abort [
+             "Manifest is missing or couldn't be read.",
+             "The Manifest is kind of a big deal.",
+             "Maybe you're using a gem packaged by a linux project.",
+             "It seems like they enjoy breaking other people's code."
+             ].join "\n" unless manifest
+
       s.name                 = name
       s.version              = version if version
       s.summary              = summary
@@ -352,16 +438,15 @@ class Hoe
       s.homepage             = Array(url).first
       s.rubyforge_project    = rubyforge_name
       s.description          = description
-      s.files = files        = File.read_utf("Manifest.txt").split(/\r?\n\r?/)
+      s.files                = manifest
       s.executables          = s.files.grep(/^bin/) { |f| File.basename(f) }
       s.bindir               = "bin"
       s.require_paths        = dirs unless dirs.empty?
       s.rdoc_options         = ['--main', readme_file]
-      s.has_rdoc             = true
       s.post_install_message = post_install_message
       s.test_files           = Dir[*self.test_globs]
 
-      missing "Manifest.txt" if files.empty?
+      missing "Manifest.txt" if s.files.empty?
 
       case author
       when Array
@@ -381,7 +466,6 @@ class Hoe
       s.extra_rdoc_files += s.files.grep(/txt$/)
       s.extra_rdoc_files.reject! { |f| f =~ %r%^(test|spec|vendor|template|data|tmp)/% }
       s.extra_rdoc_files += @extra_rdoc_files
-
     end
 
     unless self.version then
@@ -473,10 +557,19 @@ class Hoe
       }]
       desc     = sections.values_at(*description_sections).join("\n\n")
       summ     = desc.split(/\.\s+/).first(summary_sentences).join(". ")
+      urls     = parse_urls(readme[1])
 
+      self.urls        ||= urls
       self.description ||= desc
       self.summary     ||= summ
-      self.url         ||= readme[1].gsub(/^\* /, '').split(/\n/).grep(/\S+/)
+      self.url         ||= case urls
+                           when Hash then
+                             urls["home"] || urls["repo"] || urls.values.first
+                           when Array then
+                             urls
+                           else
+                             raise "unknown urls format: #{urls.inspect}"
+                           end
     else
       missing readme_file
     end
@@ -490,6 +583,33 @@ class Hoe
                      end
   end
 
+  ##
+  # Parse the urls section of the readme file. Returns a hash or an
+  # array depending on the format of the section.
+  #
+  #     label1 :: url1
+  #     label2 :: url2
+  #     label3 :: url3
+  #
+  # vs:
+  #
+  #     * url1
+  #     * url2
+  #     * url3
+  #
+  # The hash format is preferred as it will be used to populate gem
+  # metadata. The array format will work, but will warn that you
+  # should update the readme.
+
+  def parse_urls text
+    lines = text.gsub(/^\* /, '').split(/\n/).grep(/\S+/)
+    if lines.first =~ /::/ then
+      Hash[lines.map { |line| line.split(/\s*::\s*/) }]
+    else
+      lines
+    end
+  end
+
   ##
   # Load activated plugins by calling their define tasks method.
 
@@ -505,8 +625,10 @@ class Hoe
 
       begin
         send "define_#{plugin}_tasks"
-      rescue NoMethodError => e
-        warn "warning: couldn't activate the #{plugin} plugin, skipping"
+      rescue NoMethodError
+        warn "warning: couldn't activate the #{plugin} plugin, skipping" if
+          Rake.application.options.trace or $DEBUG
+
         bad << plugin
         next
       end
@@ -561,6 +683,20 @@ class Hoe
     require_rubygems_version '>= 1.3.1'
   end
 
+  ##
+  # Is a plugin activated? Used for guarding missing plugins in your
+  # hoe spec:
+  #
+  #   Hoe.spec "blah" do
+  #     if plugin? :enhancement then
+  #       self.enhancement = true # or whatever...
+  #     end
+  #   end
+
+  def plugin? name
+    self.class.plugins.include? name
+  end
+
   ##
   # Finalize configuration
 
@@ -610,9 +746,9 @@ class Hoe
   end
 
   ##
-  # Load or create a default config and yield it
+  # Loads ~/.hoerc and yields the configuration and its path
 
-  def with_config # :nodoc:
+  def with_config
     rc = File.expand_path("~/.hoerc")
     exists = File.exist? rc
     config = exists ? YAML.load_file(rc) : {}
