@@ -18,12 +18,16 @@ require 'rbconfig'
 # +FORCE_PLATFORM+ (instead of default Gem::Platform::CURRENT)
 
 module Hoe::Inline
+  def initialize_inline
+    dependency "RubyInline", "~> 3.9"
+
+    clean_globs << File.expand_path("~/.ruby_inline")
+  end
+
   ##
   # Define tasks for plugin.
 
   def define_inline_tasks
-    extra_deps  << ['RubyInline', "~> 3.8.4"]
-    clean_globs << File.expand_path("~/.ruby_inline")
     task :test => :clean
 
     if ENV['INLINE'] then
@@ -33,7 +37,7 @@ module Hoe::Inline
       if defined?(Inline) then
         directory 'lib/inline'
 
-        dlext = Config::CONFIG['DLEXT']
+        dlext = RbConfig::CONFIG['DLEXT']
 
         Inline.registered_inline_classes.each do |cls|
           name = cls.name.gsub(/::/, '')
