@@ -29,7 +29,7 @@ module Hoe::Package
   # Initialize variables for plugin.
 
   def initialize_package
-    self.need_tar ||= true
+    self.need_tar ||= false
     self.need_zip ||= false
   end
 
@@ -37,14 +37,14 @@ module Hoe::Package
   # Define tasks for plugin.
 
   def define_package_tasks
-    Gem::PackageTask.new spec do |pkg|
-      prerelease_version
+    prerelease_version
 
+    Gem::PackageTask.new spec do |pkg|
       pkg.need_tar = @need_tar
       pkg.need_zip = @need_zip
     end
 
-    desc 'Install the package as a gem.'
+    desc 'Install the package as a gem. (opt. NOSUDO=1)'
     task :install_gem => [:clean, :package, :check_extra_deps] do
       install_gem Dir['pkg/*.gem'].first
     end
@@ -54,8 +54,6 @@ module Hoe::Package
 
     # no doco, invisible hook
     task :prerelease do
-      prerelease_version
-
       abort "Fix your version before you release" if spec.version =~ /borked/
     end
 
@@ -68,16 +66,30 @@ module Hoe::Package
     desc "Sanity checks for release"
     task :release_sanity do
       v = ENV["VERSION"] or abort "Must supply VERSION=x.y.z"
+
+      pre = ENV['PRERELEASE'] || ENV['PRE']
+      v += ".#{pre}" if pre
+
       abort "Versions don't match #{v} vs #{version}" if v != version
     end
   end
 
+  ##
+  # Returns the path used for packaging. Convenience method for those
+  # that need to write a package hook.
+
+  def pkg_path
+    "pkg/#{spec.full_name}"
+  end
+
   ##
   # Install the named gem.
 
   def install_gem name, version = nil
+    should_not_sudo = Hoe::WINDOZE || ENV["NOSUDO"] || File.writable?(Gem.dir)
+
     gem_cmd = Gem.default_exec_format % 'gem'
-    sudo    = 'sudo '                  unless Hoe::WINDOZE
+    sudo    = 'sudo '                  unless should_not_sudo
     local   = '--local'                unless version
     version = "--version '#{version}'" if     version
     sh "#{sudo}#{gem_cmd} install #{local} #{name} #{version}"
