@@ -16,6 +16,7 @@ module Hoe::Test
   SUPPORTED_TEST_FRAMEWORKS = {
     :testunit => "test/unit",
     :minitest => "minitest/autorun",
+    :none     => nil,
   }
 
   ##
@@ -36,6 +37,11 @@ module Hoe::Test
 
   attr_accessor :testlib
 
+  ##
+  # Optional: Additional ruby to run before the test framework is loaded.
+
+  attr_accessor :test_prelude
+
   ##
   # Optional: RSpec dirs. [default: %w(spec lib)]
 
@@ -52,6 +58,7 @@ module Hoe::Test
   def initialize_test
     self.multiruby_skip ||= []
     self.testlib        ||= :testunit
+    self.test_prelude   ||= nil
     self.rspec_dirs     ||= %w(spec lib)
     self.rspec_options  ||= []
   end
@@ -91,29 +98,7 @@ module Hoe::Test
     end
 
     if File.directory? "spec" then
-      found = true
-      begin
-        require 'spec/rake/spectask'     # rspec 1
-
-        desc "Run all specifications"
-        Spec::Rake::SpecTask.new(:spec) do |t|
-          t.libs = self.rspec_dirs
-          t.spec_opts = self.rspec_options
-        end
-      rescue LoadError
-        begin
-          require 'rspec/core/rake_task' # rspec 2
-
-          desc "Run all specifications"
-          RSpec::Core::RakeTask.new(:spec) do |t|
-            t.rspec_opts = self.rspec_options
-            t.rspec_opts << "-I#{self.rspec_dirs.join(":")}" unless
-              rspec_dirs.empty?
-          end
-        rescue LoadError
-          found = false
-        end
-      end
+      found = try_loading_rspec2 || try_loading_rspec1
 
       if found then
         default_tasks << :spec
@@ -125,6 +110,25 @@ module Hoe::Test
     desc 'Run the default task(s).'
     task :default => default_tasks
 
+    unless default_tasks.empty? then
+      ##
+      # This is for Erik Hollensbe's rubygems-test project. Hoe is
+      # test-happy, so by using this plugin you're already testable. For
+      # more information, see: <https://github.com/erikh/rubygems-test>
+      # and/or <http://www.gem-testers.org/>
+
+      self.spec.files += [".gemtest"]
+
+      pkg  = pkg_path
+      turd = "#{pkg}/.gemtest"
+
+      file turd do
+        touch turd
+      end
+
+      file "#{pkg}.gem" => turd
+    end
+
     desc 'Run ZenTest against the package.'
     task :audit do
       libs = %w(lib test ext).join(File::PATH_SEPARATOR)
@@ -136,20 +140,65 @@ module Hoe::Test
   # Generate the test command-line.
 
   def make_test_cmd multi = false # :nodoc:
-    framework = SUPPORTED_TEST_FRAMEWORKS[testlib]
-    raise "unsupported test framework #{testlib}" unless framework
+    unless SUPPORTED_TEST_FRAMEWORKS.has_key?(testlib)
+      raise "unsupported test framework #{testlib}"
+    end
 
-    tests = ["rubygems", framework] +
-      test_globs.map { |g| Dir.glob(g) }.flatten
+    framework = SUPPORTED_TEST_FRAMEWORKS[testlib]
+
+    tests = ["rubygems"]
+    tests << framework if framework
+    tests << test_globs.sort.map { |g| Dir.glob(g) }
+    tests.flatten!
     tests.map! {|f| %(require "#{f}")}
 
+    tests.insert 1, test_prelude if test_prelude
+
     cmd = "#{Hoe::RUBY_FLAGS} -e '#{tests.join("; ")}' -- #{FILTER}"
 
     if multi then
+      multiruby_skip << "mri_trunk" if multiruby_skip.include? "1.9"
       ENV['EXCLUDED_VERSIONS'] = multiruby_skip.join ":"
       cmd = "-S multiruby #{cmd}"
     end
 
     cmd
   end
+
+  ##
+  # Attempt to load RSpec 2, returning true if successful
+
+  def try_loading_rspec2
+    require 'rspec/core/rake_task'
+
+    desc "Run all specifications"
+    RSpec::Core::RakeTask.new(:spec) do |t|
+      t.rspec_opts = self.rspec_options
+      t.rspec_opts << "-I#{self.rspec_dirs.join(":")}" unless
+      rspec_dirs.empty?
+    end
+
+    true
+  rescue LoadError => err
+    warn "%p while trying to load RSpec 2: %s" % [ err.class, err.message ]
+    false
+  end
+
+  ##
+  # Attempt to load RSpec 1, returning true if successful
+
+  def try_loading_rspec1
+    require 'spec/rake/spectask'
+
+    desc "Run all specifications"
+    Spec::Rake::SpecTask.new(:spec) do |t|
+      t.libs = self.rspec_dirs
+      t.spec_opts = self.rspec_options
+    end
+    true
+  rescue LoadError => err
+    warn "%p while trying to load RSpec 1: %s" % [ err.class, err.message ]
+    false
+  end
+
 end
